@@ -80,4 +80,8 @@ int meta_logical_monitor_get_scale (MetaLogicalMonitor *logical_monitor);
 
 GList * meta_logical_monitor_get_monitors (MetaLogicalMonitor *logical_monitor);
 
+gboolean meta_logical_monitor_has_neighbor (MetaLogicalMonitor  *logical_monitor,
+                                            MetaLogicalMonitor  *neighbor,
+                                            MetaScreenDirection  neighbor_dir);
+
 #endif /* META_LOGICAL_MONITOR_H */
